/***************************************************************************
                          tcalcbutton.cpp  -  description
                             -------------------
    begin                : ?ro lis 6 2002
    copyright            : (C) 2002 by Piotr Gridniew
    email                : gridniew@x4.pl
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#include "qpainter.h"

#include "pgcalc.h"
#include "tcalcbutton.h"
#include "keycodetr.h"

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
TCalcButton::TCalcButton( TCalc* Owner )
{
   _owner = Owner;
   _left = 0;
   _top = 0;
   _width = 0;
   _height = 0;
   _name = NULL;
}

TCalcButton::~TCalcButton()
{
}

void TCalcButton::setBounds( int Left, int Top, int Width, int Height )
{
   _left = Left;
   _top = Top;
   _width = Width;
   _height = Height;
}

void TCalcButton::setName( const char* Name )
{
   if( _name )
   {
      delete _name;
   }
   _name = new char[ strlen( Name ) + 1 ];
   memcpy( _name, Name, strlen( Name ) + 1 );
}

void TCalcButton::setKey( int Key )
{
   _key = Key;
}

void TCalcButton::setChr( unsigned char Chr )
{
   _chr = Chr;
}

void TCalcButton::setActions( TButtonAction Action, TButtonAction Mode1Action,
                              TButtonAction Mode2Action, TButtonAction AlphaAction )
{
   _action = Action;
   _mode1action = Mode1Action;
   _mode2action = Mode2Action;
   _alphaaction = AlphaAction;
}


void TCalcButton::drawPressed( void )
{
   QPainter p( _owner );
   QImage i = _owner->image().copy( _left, _top, _width - 1, _height - 1 );
   p.drawImage( _left + 1, _top + 1, i );
}

void TCalcButton::drawNormal( void )
{
   QPainter p( _owner );
   QImage i = _owner->image().copy( _left, _top, _width, _height );
   p.drawImage( _left, _top, i );
}
//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
TCalcButtonList::TCalcButtonList( void )
{
   _items = NULL;
   _count = 0;
}

TCalcButtonList::~TCalcButtonList()
{
   if( _count && _items )
   {
      Clear();
   }
}

void TCalcButtonList::Add( TCalcButton* Button )
{
   TCalcButton** tmp;

   if( Button )
   {
      tmp = new TCalcButton*[ _count + 1 ];
      if( _count && _items )
      {
         int j;
         for( j = 0; j < _count; j++ )
         {
            tmp[ j ] = _items[ j ];
         }
         delete _items;
      }
      tmp[ _count ] = Button;
      _items = tmp;
      _count++;
   }
}

void TCalcButtonList::Clear( void )
{
   int j;

   for( j = 0; j < _count; j++ )
   {
      if( _items[ j ] )
      {
         delete _items[ j ];
      }
   }
   delete _items;
   _items = NULL;
   _count = 0;
}

int TCalcButtonList::count( void )
{
   return _count;
}

TCalcButton* TCalcButtonList::item( int index )
{
   TCalcButton* b = NULL;

   if( index >= 0 && index < _count )
   {
      b = _items[ index ];
   }
   return b;
}

TCalcButton* TCalcButtonList::itemAt( int x, int y )
{
   TCalcButton* button = NULL;
   int j = 0;
   int l, t, r, b;

   for( j = 0; j < _count && !button; j++ )
   {
      l = _items[ j ]->left();
      t = _items[ j ]->top();
      r = l + _items[ j ]->width();
      b = t + _items[ j ]->height();
      
      if( x >= l && x <= r && y >= t && y <= b )
      {
         button = _items[ j ];
      }
   }
   return button;
}

TCalcButton* TCalcButtonList::getByKeyCode( int keycode )
{
   TCalcButton* button = NULL;
   int j = 0;

   for( j = 0; j < _count && !button; j++ )
   {
      if( _items[ j ]->key() == keycode )
      {
         button = _items[ j ];
      }
   }
   return button;
}

//----------------------------------------------------------------------------
//
//
//
//----------------------------------------------------------------------------
TButtonAction TranslateAction( const char* str )
{
   TButtonAction action;
   int j;

   j = 0;
   action = baUnknown;
   while( BtnDesc[ j ].action != baUnknown && action == baUnknown )
   {
      if( strcmp( BtnDesc[ j ].name, str ) == 0 )
      {
         action = BtnDesc[ j ].action;
      }
      else
      {
         j++;
      }
   }
   return action;
}

//----------------------------------------------------------------------------

int TranslateKey( const char* str )
{
   int keycode;
   unsigned int j;
   bool numeric;

   if( strlen( str ) == 0 )
   {
      return 0;
   }

   if( strlen( str ) > 1 )
   {   
      for( j = 0, numeric = true; j < strlen( str ) && numeric; j++ )
      {
         if( str[ j ] >= '0' && str[ j ] <= '9' )
         {
         }
         else
         {
            numeric = false;
         }
      }
   }
   else
   {
      numeric = false;
   }
   if( numeric )
   {
      int x;
      sscanf( str, "%d", &x );
      keycode = WinKeyToLinux( x );
      if( keycode == -1 )
      {
         keycode = x;
      }
   }
   else
   {
      keycode = str[ 0 ];
   }
   return keycode;
}
